@@ -92,7 +92,9 @@ impl LspUrlMap {
       let url = if specifier.scheme() == "file" {
         specifier.clone()
       } else {
-        let specifier_str = if specifier.scheme() == "data" {
+        let specifier_str = if specifier.scheme() == "asset" {
+          format!("deno:asset{}", specifier.path())
+        } else if specifier.scheme() == "data" {
           let data_url = DataUrl::process(specifier.as_str())
             .map_err(|e| uri_error(format!("{:?}", e)))?;
           let mime = data_url.mime_type();
