@@ -12,7 +12,9 @@ unitTest(async function sendAsyncStackTrace() {
     await Deno.read(rid, buf);
     unreachable();
   } catch (error) {
-    const s = error.stack.toString();
+    assert(error instanceof Error);
+    const s = error.stack?.toString();
+    assert(s);
     console.log(s);
     assertStringIncludes(s, "opcall_test.ts");
     assertStringIncludes(s, "read");
@@ -25,7 +27,7 @@ unitTest(async function sendAsyncStackTrace() {
 
 declare global {
   namespace Deno {
-    // deno-lint-ignore no-explicit-any
+    // deno-lint-ignore no-explicit-any, no-var
     var core: any;
   }
 }
@@ -33,8 +35,7 @@ declare global {
 unitTest(async function opsAsyncBadResource() {
   try {
     const nonExistingRid = 9999;
-    await Deno.core.opAsync(
-      "op_read_async",
+    await Deno.core.read(
       nonExistingRid,
       new Uint8Array(0),
     );
