@@ -3,6 +3,7 @@ import {
   assert,
   assertEquals,
   assertStringIncludes,
+  assertThrows,
   unitTest,
 } from "./test_util.ts";
 
@@ -56,7 +57,24 @@ unitTest(async function responseFormData() {
   assertEquals([...formData], [...input]);
 });
 
-unitTest(function customInspectFunction(): void {
+unitTest(function responseInvalidInit() {
+  // deno-lint-ignore ban-ts-comment
+  // @ts-expect-error
+  assertThrows(() => new Response("", 0));
+  assertThrows(() => new Response("", { status: 0 }));
+  // deno-lint-ignore ban-ts-comment
+  // @ts-expect-error
+  assertThrows(() => new Response("", { status: null }));
+});
+
+unitTest(function responseNullInit() {
+  // deno-lint-ignore ban-ts-comment
+  // @ts-expect-error
+  const response = new Response("", null);
+  assertEquals(response.status, 200);
+});
+
+unitTest(function customInspectFunction() {
   const response = new Response();
   assertEquals(
     Deno.inspect(response),
